import { CommonModule } from "@angular/common";
import {
  ChangeDetectionStrategy,
  Component,
  Input,
  NgModule,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { FormsModule } from "@angular/forms";
import {
  SohoAutoCompleteModule,
  SohoBusyIndicatorDirective,
  SohoButtonModule,
  SohoComponentsModule,
  SohoFileUploadAdvancedModule,
  SohoModalDialogService,
  SohoToastModule,
} from "@infor/sohoxi-angular";

import {
  ILanguage,
  IDialogResult,
  IIonApiRequestOptions,
  IWidgetAction,
  IWidgetComponent,
  IWidgetContext,
  IWidgetInstance,
  WidgetMessageType,
  WidgetState,
} from "lime";
import { FAIModalComponent } from "./modal";

@Component({
  template: `
    <div #fileupload>
      <div
        soho-busyindicator
        class="full-width full-height scrollable-flex"
        [blockUI]="true"
        role="main"
      >
        <div class="toolbar" role="toolbar">
          <div class="more">
            <button class="btn-actions" type="button">
              <svg
                class="icon"
                focusable="false"
                aria-hidden="true"
                role="presentation"
              >
                <use href="#icon-more"></use>
              </svg>
              <span class="audible">More Actions</span>
            </button>
          </div>
        </div>

        <div>
          <b>{{ language?.get("widgetVersion") }}</b>: 3.1
          <br />
          <br />

          <b>{{ language?.get("desHowToUse")}}</b><br />
          {{ language?.get("desUploadFileUsage")}}
          <br />
          <br />

          <b>{{ language?.get("desWidgetDescription")}}</b><br />
         {{ language?.get("desWidgetUsage")}}
        </div>
      </div>
    </div>
  `,
  providers: [FAIModalComponent],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class FileUploadAIComponent implements IWidgetComponent, OnInit {
  widgetContext: IWidgetContext;
  widgetInstance: IWidgetInstance;
  @ViewChild(SohoBusyIndicatorDirective, { static: true })
  @ViewChild("fileupload", { read: ViewContainerRef, static: false })
  busyIndicator: SohoBusyIndicatorDirective;
  title: string; 
  sohodialogService: SohoModalDialogService;
  idsModalComponent: FAIModalComponent;
  model: unknown;
  placeholder: ViewContainerRef;
  xssSource: SohoAutoCompleteSource = [];
  openedAsModal = false;
  language: ILanguage;


  
  constructor(
    sohodialogService: SohoModalDialogService,
    private readonly viewRef: ViewContainerRef,
    idsModalComponent: FAIModalComponent
    ) 
  {
    this.sohodialogService = sohodialogService;
    this.placeholder = viewRef;
    this.idsModalComponent = idsModalComponent;
  }
  
  ngOnInit() {
    this.idsModalComponent.widgetContext = this.widgetContext;
    this.idsModalComponent.widgetInstance = this.widgetInstance;
    this.language = this.widgetContext.getLanguage();
    this.title = this.language.get("desConfigureWidget");
    if (!this.openedAsModal) {
      this.widgetInstance.actions[0].execute = () => this.openModal();
    }
  }
  openModal() {
    this.setBusy(true);
    this.callDataObjects();
  }
  private setBusy(isBusy: boolean): void {
    this.widgetContext.setState(
      isBusy ? WidgetState.busy : WidgetState.running
    );
  }
  callDataObjects() {
    const request = this.creatObjectSchemaRequest();
    this.widgetContext.executeIonApiAsync<any>(request).subscribe(
      (response) => {
        this.setBusy(false);
        let objectSchemaData = response.data;
        let dataSchemaResponse = JSON.parse(objectSchemaData);
        const tempSchemaObjects: string[] = new Array();
        for (
          let i = 0;
          i < dataSchemaResponse.objects.length &&
          dataSchemaResponse.objects[i] != null;
          i++
        ) {
          const name = dataSchemaResponse.objects[i].name;
          tempSchemaObjects.push(name);
        }
        this.xssSource = tempSchemaObjects;
        this.setBusy(false);
        const dialog = this.sohodialogService
          .modal(FAIModalComponent, this.placeholder)
          .title(this.language.get("desUploadFileTitle"))
          .afterClose((result: IDialogResult) => {
            const message: string = result
              ? (result.value as string)
              : this.language.get("desDialogCancelled");
          });

        dialog
          .apply((component: FAIModalComponent) => {
            component.modalRef = dialog;
            component.widgetContext = this.widgetContext;
            component.widgetInstance = this.widgetInstance;
            component.xssSource = this.xssSource;
          })
          .open();
        if (response.status == 201 || response.status == 200) {
          this.setBusy(false);
        }
      },
      (error) => {
        this.setBusy(false);
        var message = this.getErrorMessage(error);

        this.setBusy(false);
        this.widgetContext.showWidgetMessage({
          message: message,
          type: WidgetMessageType.Error,
        });
      }
    );
  }
  getErrorMessage(error: any): string {
    var message = "";
    if (error.status == 401) {
      message = this.language.get("desFailedToGetObjectSchema") + " : " + this.language.get("desUnAuthorized");
    } else if (error.status == 404) {
      message = this.language.get("desFailedToGetObjectSchema") + " : " + this.language.get("desNotFound");
    } else if (error.status == 405) {
      message = this.language.get("desFailedToGetObjectSchema") + " : " + this.language.get("desMethodNotAllowed");
    } else if (error.status == 412) {
      message = this.language.get("desFailedToGetObjectSchema") + " : " + this.language.get("desPreconditionFailed");
    } else if (error.status == 415) {
      message = this.language.get("desFailedToGetObjectSchema") + " : " + this.language.get("desUnsupportedMediaType");
    } else if (error.status == 500) {
      message = this.language.get("desFailedToGetObjectSchema") + " : " + this.language.get("desInternalServerError");
    } else {
      message = this.language.get("desUnableToGetResponse");
    }

    return message;
  }
  creatObjectSchemaRequest(headers?: object): IIonApiRequestOptions {
    if (!headers) {
      headers = { Accept: "*/*", "Content-type": "application/xml" };
    }
    let jsonData: any;
    const url = "IONSERVICES/datacatalog/v1/object/list";
    const request: IIonApiRequestOptions = {
      method: "GET",
      url: url,
      cache: false,
      headers: headers,
      responseType: "text" as const,
    };

    return request;
  }
}

@NgModule({
  imports: [
    CommonModule,
    FormsModule,
    SohoComponentsModule,
    SohoButtonModule,
    SohoAutoCompleteModule,
    SohoFileUploadAdvancedModule,
    SohoToastModule,
  ],
  providers: [FAIModalComponent],
  declarations: [FileUploadAIComponent, FAIModalComponent],
  entryComponents: [FileUploadAIComponent, FAIModalComponent],
})
export class FileAIModule {}

export const getActions = (context: IWidgetContext): IWidgetAction[] => {
  const lang = context.getLanguage();
  return [
    {
      isPrimary: true,
      standardIconName: "#icon-folder",
      text: lang.get("fileUpload") ,
      
    },
  ];
};
